#!/usr/bin/python

import os, json, threading, select, socket
import players
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
from urllib import unquote_plus
try:
  from config import file_browser_path, web_remote_control, remote_guide
  from browser import seen
  import tvguide
except ImportError:
  file_browser_path = os.path.abspath(".")
  web_remote_control = ('', 9009)

content_types = dict(
  js = "text/javascript",
  css = "text/css",
  gif = "image/gif",
  png = "image/png",
  jpg = "image/jpg"
)

def hms0(seconds, percent=0.0):
    print seconds, percent
    if not seconds or percent is None or percent>=100.0:
      return ""
    value = seconds*(100.0-percent)/100.0
    return "%d:%02d:%02d" % (   
      value/3600,
      value/60%60,
      value % 60
    )

class server_class(HTTPServer):
  timeout = 0.01

class handler_class(BaseHTTPRequestHandler):
  CHANNEL_TEMPLATE = '''\
        <label>
            <input name="channel" id="ch-%(id)s" value="%(id)s" data-theme="d" type="radio">%(name)s
        </label>
  '''
  IMG_TEMPLATE = '<img src="%s" alt=""/>'
  GUIDE_TEMPLATE = '''\
    <div data-role="collapsible" data-collapsed="false" data-theme="b" data-content-theme="d">
        <h4>%(name)s</h4>
        <ul data-role="listview">
            <li>
              <a href="#" data-rel="back" onclick="change_channel(%(channel)s);">%(guide)s</a>
              <form>
                <input name="pos-%(channel)s" value="%(progress)d" min="0" max="100" disabled="disabled" data-highlight="true" type="range">
                %(img)s
              </form>
            </li>
        </ul>
    </div>
  '''
  PROFILE_TEMPLATE = '''\
    <a href="#" id="f%s" data-role="button" data-mini="true" data-theme="%s">P%s</a>
  '''
  root = os.path.abspath(os.path.dirname(__file__))
  def fix_path(self, arg):
      filepath = os.path.abspath(os.path.join(file_browser_path, arg))
      if filepath.startswith(file_browser_path):
        return filepath
      return None
  def send_file(self, filename, content_type=None, **data):
      if content_type==None:
        content_type=="text/plain"
        for key, value in content_types.items():
          if filename.endswith("."+key):
            content_type = value
            break
      self.send_response(200)
      self.send_header("Content-type", "%s; charset=UTF-8" % content_type)
      self.end_headers()
      filename = os.path.abspath(os.path.join(self.root, filename))
      if filename.startswith(self.root):
        if data:
          self.wfile.write(open(filename).read() % data)
        else:
          self.wfile.write(open(filename).read())
      else:
        self.wfile.write("PERMISSION DENIED!")
      self.wfile.close()
  def send_data(self, data):
      self.send_response(200)
      self.send_header("Content-type", "text/plain; charset=UTF-8")
      self.send_header("Location", "/")
      self.end_headers()
      self.wfile.write(json.dumps(data))
      self.wfile.close()
  def get_channels(self):
      return enumerate([x.name for x in self.player.urls])
  def channel_name(self):
      ch_name = self.player.urls[self.player.channel-1].name
      if self.player.last_message:
        ch_name += ": " + self.player.last_message
      return ch_name
  def source(self):
      url = self.player.urls[self.player.channel-1]
      surl = url(self.player.source)
      class ret:
        name = surl.name
        source = surl.source
      if url==surl:
        ret.name = ""
      return ret
  def do_GET(self):
      if self.path=="/" or self.path.startswith("/?"):
        channels = [
          self.CHANNEL_TEMPLATE % dict(id=key+1, name=value)
          for key, value in self.get_channels()
        ]
        profiles = []
        print "PROFILE", self.player.profile
        for i in range(9):
          if i+1==self.player.profile:
            theme = "e"
          else:
            theme = "a"
          profiles.append(self.PROFILE_TEMPLATE % (
            i+1, theme, i+1
          ))
        source = self.source()
        self.send_file("index.html",
          channels = "\n".join(channels),
          channel_name = self.channel_name(),
          source_name = source.name,
          source = source.source,
          profiles = '\n'.join(profiles),
          volume = self.player.volume,
          position = self.player.last_pos,
          content_length = hms0(self.player.content_length),
          content_remains = hms0(
            self.player.content_length, self.player.last_pos
          ),
          hostname = socket.gethostname()
        )
      elif self.path in ["/client.js", "/style.css", "/favicon.png"]:
        self.send_file(self.path[1:])
      elif self.path.startswith("/js/"):
        self.send_file(os.path.join(".", self.path[1:]))
      elif self.path=="/tvguide.html":
        content = []
        for ch, url in enumerate(self.player.urls):
          guide = url.find_guide()
          if guide:
            msg = tvguide.profi_program.current(guide.name)
            if msg and msg.name:
              if msg.img:
                img = self.IMG_TEMPLATE % msg.img
              else:
                img = ''
              content.append(self.GUIDE_TEMPLATE % dict(
                channel = ch+1,
                name = url.name,
                guide = msg.name,
                img = img,
                progress = msg.progress()*100
              ))
        self.send_file(self.path[1:],
          content='\n'.join(content),
          remote_guide = remote_guide,
          hostname=socket.gethostname()
        )
      elif self.path.count("/")>=2:
        path = self.path.rsplit("?", 1)[0]
        empty, cmd, arg = path.split("/", 2)
        print "Mobile cmd: %s(%s)" % (cmd, arg)
        source = self.source()
        reply = dict(
          volume = self.player.volume,
          mute = self.player.mute,
          position = self.player.last_pos,
          content_length = hms0(self.player.content_length),
          content_remains = hms0(
            self.player.content_length, self.player.last_pos
          ),
          profile = self.player.profile,
          channel = self.player.channel,
          channel_name = self.channel_name(),
          source_name = source.name,
          source = source.source
        )
        if cmd=="noop" or cmd=="status":
          pass
        elif cmd=="volume":
          self.player.change_volume(volume=int(arg))
          reply['volume'] = self.player.volume
        elif cmd=="mute":
          self.player.change_volume(mute=arg=='on')
          reply['mute'] = self.player.mute
        elif cmd=="profile":
          self.player.change_profile(int(arg))
          reply['reload_page'] = True
        elif cmd=="channel":
          if arg=="prev":
            self.player.change_channel(direction=-1, player=players.default)
          elif arg=="next":
            self.player.change_channel(direction=1, player=players.default)
          elif arg=="last":
            self.player.change_channel(
              channel=self.player.last_channel,
              player=players.default
            )
          else:
            self.player.change_channel(
              channel=int(arg),
              player=players.default
            )
        elif cmd=="change_channel":
          self.player.change_channel(channel=int(arg), player=players.default)
          reply['location'] = '/'
        elif cmd=="video":
          self.player.source += 1
          self.player.change_channel()
        elif cmd=="audio":
          self.player.change_track(+1)
        elif cmd=="subtitle":
          self.player.change_subtitle()
        elif cmd=="aspect":
          self.player.change_aspect()
        elif cmd=="position":
          if arg=="pause":
            self.player.player.pause()
          elif arg.startswith("rew"):
            self.player.change_position(-int(arg[3:]))
          elif arg.startswith("ff"):
            self.player.change_position(int(arg[2:]))
          #else:
          #  self.player.change_position(seek=int(arg))
        elif cmd=="exit":
          if self.player.onscreen==self.player.youtube:
            self.player.youtube.mainbox.hide()
            self.player.onscreen = self.player.player
        elif cmd=="quit":
          self.player.destroy(save_size=True)
        elif cmd=="fullscreen":
          self.player.change_fullscreen(1)
        elif cmd=="browser":
          counter = {}
          filelist = ['../']
          filepath = self.fix_path(arg.rstrip("/"))
          if filepath:
            for fn in sorted(os.listdir(filepath)):
              fullpath = os.path.join(filepath, fn)
              if fn==".." or os.path.isdir(fullpath):
                filelist.append("%s/" % fn)
              else:
                filelist.append(fn)
                counter[fn] = seen.count(fullpath)
          reply['filelist'] = filelist
          reply['filepath'] = arg
          reply['counter'] = counter
        elif cmd=="youtube" and self.player.onscreen!=self.player.youtube:
          print "Youtube seach for:", arg
          self.player.onscreen = self.player.youtube
          self.player.youtube.search(unquote_plus(arg))
        elif cmd=="play":
          print "PLAY:", arg
          filename = self.fix_path(unquote_plus(arg))
          if filename:
            seen.inc(filename)
            seen.save()
            self.player.play_files([os.path.basename(filename)], [filename])
        self.send_data(reply)
      else:
        self.send_response(500)
        self.send_header("Content-type", "text/plain; charset=UTF-8")
        self.end_headers()

if __name__ == "__main__":
  import time
  #server = thread(web_remote_control, testing_channels, None)
  httpd = server_class(web_remote_control, handler_class)
  for i in range(600):
    print httpd.handle_request()
    print "Sleep %d" % i
  #server.stop()
  print "Stopped"
