# -*- coding: UTF-8 -*-

import sys, socket, urllib, locale

FILES = {
  '': 'text/html',
  'playlist.m3u': 'application/x-mpegurl',
  'layout.css': 'text/css',
  'controls.js': 'application/javascript'
}

TEMPLATE='''\
<!DOCTYPE html>
<html>
<head>
<title>VideoRelay</title>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<meta name="viewport" content="initial-scale=0.8, maximum-scale=0.8">
<link rel="stylesheet" type="text/css" media="all" href="layout.css" />
<script src="http://www.salstar.sk/js/jquery.min.js" type="text/javascript"></script>
<script src="controls.js" type="text/javascript"></script>
</head>
<body>
<form action="#" class="tabular" onsubmit="return send_form();">
  <input type="hidden" name="base_url" value="%s"/>
  <div>Quality:</div>
  <select name="format" onchange="change_form()">
    <optgroup label="Video formats">
      <option value="auto/b=512">Auto @512 kbps</option>
      <option value="vp8/b=512">WEBM vp8 @512 kbps</option>
      <option value="vp9/b=256">WEBM vp9 @256 kbps</option>
      <option value="ogv/b=512">OGV theora @512 kbps</option>
      <option value="x264/b=512">MP4 x264 @512 kbps</option>
      <option value="mpeg4/b=1024">MPEGTS mpeg4 @1024 kbps</option>
      <option value="rawvideo">Raw video stream</option>
    </optgroup>
    <optgroup label="Audio formats">
      <option value="oga/b=128">OGA vorbis @128 kbps</option>
      <option value="mp3/b=128">MPEG mp3 @128 kbps</option>
      <option value="rawaudio">Raw audio stream</option>
    </optgroup>
  </select>
  <br/>
  <div>Resolution:</div>
  <select name="resolution" onchange="change_form()">
    <option value="w=480/h=360">480x360 [4:3]</option>
    <option value="w=640/h=360">640x360 [16:9]</option>
    <option value="w=400/h=300">400x300 [4:3]</option>
    <option value="w=533/h=300">533x300 [16:9]</option>
    <option value="w=320/h=240">320x240 [4:3]</option>
    <!-- <option value="w=800/h=480">800x600</option> -->
  </select>
  <br/>
%s
<div>&nbsp;</div>
<input type="submit" value="Play!" onclick="return play_media()"/>
<input type="submit" value="Play in new window!"/>
<input type="submit" value="Stop!" onclick="return stop_media()"/>
</form>
<form action="#" onsubmit="return play_media();">
<a href="#" id="aurl">Target URL:</a>
<input id="url" type="text" size="80" value=""/>
</form>
<hr/>
<div id="player">
</div>
<hr/>
<textarea readonly="readonly" onclick="return refresh_info()">
Click to refresh info page!
</textarea>
</body>
</html>
'''

channels_py_example = '''\
# -*- coding: UTF-8 -*-

def multicast(*args):
    return list(args)
    
CHANNELS = [
  [u"Channel name", "rtp://232.10.10.10:1234/", "GROUP"],
]
'''

try:
  from channels import CHANNELS
except ImportError, err:
  if str(err).startswith("No module"):
    print "ERROR: no channels.py file. Example file:"
    print channels_py_example
    sys.exit()
  raise

class page(object):
  def __init__(self):
      locale.setlocale(locale.LC_COLLATE, 'sk_SK.utf8')
      CHANNELS.sort(locale.strcoll, key=lambda x: x[0])
      # get channel list
      self.ch_groups = ['Favorites', 'TV', 'HDTV', 'Radio']
      for ch_name, ch_url, ch_grp in CHANNELS:
        if ch_grp=="TEST":
          continue
        if ch_grp not in self.ch_groups:
          self.ch_groups.append(ch_grp)
      self.channels = CHANNELS

  def rtp2url(self, ch_name, ch_url):
      ch_name = ch_name.replace("/", "-").encode("UTF-8")
      ch_ip = ch_url.split("/")[2].split(":")[0]
      url = "ip=%s//%s" % (ch_ip, urllib.quote_plus(ch_name))
      return ch_name, url

  def index(self, prefix=""):
      ret = []
      for group in self.ch_groups:
        #ksort($chlist, SORT_LOCALE_STRING);
        ret.append('<div>%s:</div>' % group)
        ret.append(
          '''<select name="%s" class="channel" onchange="change_channel('%s')">'''
          % (group, group)
        )
        ret.append('<option value="">--- Please select one ---</option>')
        for ch_name, ch_url, ch_grp in self.channels:
          if ch_grp!=group:
            continue
          ch_name, url = self.rtp2url(ch_name, ch_url)
          ret.append('<option value="%s">%s</option>' % (url, ch_name))
        ret.append('</select><br/>')
      return TEMPLATE % (prefix, '\n'.join(ret))

  def m3u(self, prefix="", format="rawvideo"):
      ret = ["#EXTM3U"]
      for group in ['Favorites', 'TV']:
        for ch_name, ch_url, ch_grp in self.channels:
          if ch_grp!=group:
            continue
          ch_name, url = self.rtp2url(ch_name, ch_url)
          ret.append("#EXTINF:-1,%s" % ch_name)
          ret.append("%s/%s/%s" % (prefix, format, url))
      return "\n".join(ret)+"\n"

if __name__=="__main__":
  print page().index()
