#!/usr/bin/python

'''
Youtube browser module for SVPlayer

(c) 2011-2015 Jan ONDREJ (SAL) <ondrejj(at)salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
'''

import sys, os, time, socket, random, array, urllib2, cookielib
from gdata.youtube.service import YouTubeService, YouTubeVideoQuery
from urllib2 import urlopen
from PIL import Image
from StringIO import StringIO
from sgtk import *
import players
from config import fifo_filename, cookies_filename, youtube, \
                   youtube_player_rows, youtube_player_columns, \
                   recorder_path, svplayer_history, history_filename

# load youtube-dl from /usr/bin
sys.path.insert(0, "/usr/bin/youtube-dl")
sys.path.insert(1, os.path.join(os.path.dirname(__file__), "youtube-dl"))
from youtube_dl.extractor import YoutubeIE
from youtube_dl.YoutubeDL import YoutubeDL

youtube_history = svplayer_history(history_filename)

def totime(sec):
    sec = int(sec or 0)
    if sec>=3600:
      return "%d:%02d:%02d" % (sec/3600, sec/60%60, sec%60)
    return "%d:%02d" % (sec/60%60, sec%60)

def thumbnail_image(content, label_text, duration):
    box1 = Gtk.VBox()
    box1.set_border_width(2)
    
    im = Image.open(StringIO(content))
    im = im.resize((300, 180), Image.ANTIALIAS)
    buff = StringIO()
    im.save(buff, "ppm")
    loader = GdkPixbuf.PixbufLoader.new_with_type('pnm')
    loader.write(buff.getvalue())
    loader.close()
    image = Gtk.Image()
    image.set_from_pixbuf(loader.get_pixbuf())
    
    label = Gtk.Label(label_text[:25] + "\n[%ss]" % totime(duration.seconds))
    
    box1.pack_start(image)
    box1.pack_start(label)
    
    image.show()
    label.show()
    box1.show()
    return box1, label

class SvpYoutubeIE(YoutubeIE):
  _formats = dict([
    (k,v) for k,v in YoutubeIE._formats.items()
    if v.get('height', 0)<1080
  ])

class FileUrl(YoutubeDL):
  def process_info(self, info_dict):
      self.url = info_dict.get('url')

class urldl:
  def __init__(self, url, cookies):
      self.cookies = '; '.join(["%s=%s" % (x.name, x.value) for x in cookies])
      self.host, self.url = url.replace('http://', '').split('/', 1)
      self.port = 80
      # connect to host
      print "DL:", self.host, self.url, self.cookies
      self.connect()
  def connect(self):
      self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      self.s.connect((self.host, int(self.port)))
      self.s.sendall("GET /%s HTTP/1.1\r\n" % self.url)
      self.s.sendall("Host: %s\r\n" % self.host)
      self.s.sendall("Cookie: %s\r\n" % self.cookies)
      self.s.sendall("\r\n")
      f = self.s.makefile('r', 0)
      line = f.readline().strip().split(' ', 2)
      print line
      if line[1]=="200":
        list(self.read_headers(f))
        f.close()
      elif line[1]=="302":
        # watch for Location: header
        h = dict(self.read_headers(f))
        if "Location" in h:
          url = h["Location"]
          print "Redir:", url
          self.host, self.url = url.replace('http://', '').split('/', 1)
          f.close()
          self.s.shutdown(socket.SHUT_RDWR)
          self.connect()
      else:
        print "ERROR:", line
        f.close()
        self.s.shutdown(socket.SHUT_RDWR)
  def read_headers(self, f):
      while True:
        x = f.readline().rstrip("\r\n")
        if not x: break
        print x
        yield x.split(': ', 1)
  def fileno(self):
      return self.s.fileno()

CSS_DATA = """\
GtkButton
{
        background-color: #B00606;
        background-image: -gtk-gradient (linear,
                left top,
                left bottom,
                from (#b00606),
                color-stop (0.5, lighter (#b00606)),
                to (#b00505)
        );
}
GtkButton GtkLabel
{
        color: white;
}
"""

css_provider = Gtk.CssProvider()
css_provider.load_from_data(CSS_DATA)

class youtube_player:
  def __init__(self, player,
               rows=youtube_player_rows, columns=youtube_player_columns):
      #self.default_style = None
      self.rows = rows
      self.displayed_rows = rows
      self.columns = columns
      self.mainbox = Gtk.VBox()
      self.player = player
      self.related = None
      self.vbox = None
  def search(self, search_text):
      youtube_history.push(search_text)
      #socket.setdefaulttimeout(1)
      yt = YouTubeService().YouTubeQuery(YouTubeVideoQuery(
             text_query=search_text)
           )
      #socket.setdefaulttimeout(None)
      print "Youtube found:", len(yt.entry), yt.entry
      self.add_vbox(yt.entry)
      return yt.entry
  def search_related(self):
      if not self.related:
        return
      self.add_vbox(self.related)
  def search_history(self, rows=4):
      entries = []
      for search_text in youtube_history.get(0, rows):
        yt = YouTubeService().YouTubeQuery(YouTubeVideoQuery(
               text_query=search_text)
             )
        print "Youtube found:", len(yt.entry)
        if yt.entry:
          entries.extend(yt.entry[0:self.columns*3])
      # randomize order
      random.shuffle(entries)
      self.add_vbox(entries, rows)
  def update_related(self, href):
      # search for related
      parsed = dict(
        [x.split('=', 1) for x in href.split('?', 1)[1].split('&')]
      )
      #socket.setdefaulttimeout(1)
      self.related = YouTubeService().GetYouTubeRelatedVideoFeed(
                                        video_id=parsed['v']
                                      ).entry
      #socket.setdefaulttimeout(None)
  def remove_vbox(self):
      if self.vbox:
        self.mainbox.remove(self.vbox)
      self.mainbox.hide()
      self.vbox = None
  def add_vbox(self, entries, rows=None):
      self.index = 0
      self.entries = entries[:]
      self.buttons = []
      self.labels = []
      self.remove_vbox()
      self.mainbox.show()
      self.vbox = Gtk.VBox()
      self.vbox.show()
      self.mainbox.add(self.vbox)
      for y in range(rows or self.rows):
        hbox = Gtk.HButtonBox()
        hbox.show()
        self.vbox.pack_start(hbox, fill=True)
        for x in range(self.columns):
          if not entries:
            break
          entry = entries.pop(0)
          #print "Reading img:", entry.media.thumbnail[0].url
          try:
            imgcontent = urlopen(
                           entry.media.thumbnail[0].url, timeout=2
                         ).read()
          except urllib2.URLError, e:
            print "ERROR downloading thumbnail %s: %s" \
                  % (entry.media.thumbnail[0].url, e)
            imgcontent = None
          button = Gtk.Button()
          button.modify_bg(Gtk.STATE_NORMAL, COLOR_BLACK)
          button.connect("clicked", self.play_gst, entry)
          #button.connect("clicked", self.play_mplayer, entry)
          th_box, th_label = thumbnail_image(imgcontent, entry.title.text,
                               entry.media.duration)
          button.add(th_box)
          button.set_size_request(320, 240)
          button.show()
          self.buttons.append(button)
          self.labels.append(th_label)
          hbox.pack_start(button, fill=True)
      self.vbox.set_focus_chain([])
      self.displayed_rows = rows or self.rows
      self.go()
  def go(self, x=0, y=0):
      if self.buttons:
        self.buttons[self.index].get_style_context().remove_provider(
          css_provider)
        self.labels[self.index].get_style_context().remove_provider(
          css_provider)
      self.index += x + y*self.columns
      max = self.displayed_rows*self.columns
      if self.index<0:
        self.index = max+self.index
      elif self.index>=max:
        self.index -= max
      print "INDEX:", self.index, max, x, y
      if self.buttons:
        self.buttons[self.index].grab_focus()
        self.buttons[self.index].get_style_context().add_provider(
          css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
        self.labels[self.index].get_style_context().add_provider(
          css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
  def keyboard(self, widget=None, event=None, key=None):
      if key is None:
        key = Gtk.gdk.keyval_name(event.keyval)
      if (key=="Down" or key=="REW_m") and self.displayed_rows>1:
        self.go(0, 1)
        return True
      elif (key=="Up" or key=="FF_m") and self.displayed_rows>1:
        self.go(0, -1)
        return True
      elif key=="Right" or key=="FF_s" or key=="Down":
        self.go(1)
        return True
      elif key=="Left" or key=="REW_s" or key=="Up":
        self.go(-1)
        return True
      elif key=='R':
        self.download()
        return True
      return False
  def pushkey(self, key):
      self.keyboard(key=key)
  def play_vlc_fifo(self, widget, data=None):
      href = data.link[0].href
      self.remove_vbox()
      # make fifo pipe and play
      try:
        os.unlink(fifo_filename)
      except:
        pass
      os.mkfifo(fifo_filename)
      self.player(
        [data.title.text],
        ["file://" + fifo_filename],
        media=youtube,
        player=players.VLCWidget
      )
      time.sleep(1)
      os.system("youtube-dl -o %s '%s' &" % (fifo_filename, href))
      self.update_related(href)
  def download(self):
      url = self.entries[self.index].link[0].href
      os.system(
        "youtube-dl -c -o '%s/%%(title)s-%%(id)s.%%(ext)s' %s &"
        % (recorder_path, url)
      )
  def play_mplayer(self, widget, data=None):
      href = data.link[0].href
      self.remove_vbox()
      self.update_related(href)
      # store cookies
      jar = cookielib.MozillaCookieJar(cookies_filename)
      cookie_processor = urllib2.HTTPCookieProcessor(jar)
      urllib2.install_opener(urllib2.build_opener(cookie_processor))
      fd = FileUrl(dict(
             forceurl=True,
             simulate=True,
             quiet=True,
             outtmpl=u'%(id)s.%(ext)s'
           ))
      fd.add_info_extractor(SvpYoutubeIE())
      fd.download([href])
      url = fd.url
      jar.save()
      print self.player, url
      self.player(
        [data.title.text],
        [url],
        media=youtube,
        player=players.MPlayerWidget
      )
  def play_vlc(self, widget, data=None):
      href = data.link[0].href
      self.remove_vbox()
      self.update_related(href)
      # use youtube-dl to extract url and cookies
      jar = cookielib.MozillaCookieJar(cookies_filename)
      cookie_processor = urllib2.HTTPCookieProcessor(jar)
      urllib2.install_opener(urllib2.build_opener(cookie_processor))
      fd = FileUrl(dict(forceurl=True, simulate=True,
                        quiet=True, outtmpl=u'%(id)s.%(ext)s'))
      fd.add_info_extractor(SvpYoutubeIE())
      fd.download([href])
      #host, url = fd.url.replace('http://', '').split('/', 1)
      # connect to host
      conn = urldl(fd.url, jar)
      fd = conn.fileno()
      self.player(
        [data.title.text],
        [fd],
        media=youtube,
        player=players.VLCWidget,
        options={}
      )
  def play_gst(self, widget, data=None):
      href = data.link[0].href
      self.mainbox.remove(self.vbox)
      self.vbox = None
      self.mainbox.hide()
      self.update_related(href)
      # use youtube-dl to extract url and cookies
      jar = cookielib.MozillaCookieJar(cookies_filename)
      cookie_processor = urllib2.HTTPCookieProcessor(jar)
      urllib2.install_opener(urllib2.build_opener(cookie_processor))
      fd = FileUrl(dict(forceurl=True, simulate=True,
                        quiet=True, outtmpl=u'%(id)s.%(ext)s'))
      fd.add_info_extractor(SvpYoutubeIE())
      fd.download([href])
      # connect to host
      #conn = urldl(fd.url, jar)
      #fd = conn.fileno()
      req = urllib2.Request(fd.url) # create new request
      jar.add_cookie_header(req) # add cookies to request
      cookies = [
        x[1]
        for x in req.header_items()
        if x[0]=="Cookie"
      ]
      print "Playing youtube: %s [%s]" % (
        data.title.text, totime(data.media.duration.seconds)
      )
      self.player(
        [data.title.text],
        [fd.url],
        media=youtube,
        player=players.GSTWidget,
        options=dict(cookies=cookies)
      )

if __name__ == "__main__":
  print "TEST"
  yt = YouTubeService().YouTubeQuery(YouTubeVideoQuery(
         text_query=sys.argv[1])
       )
  #socket.setdefaulttimeout(None)
  print "Youtube found:", len(yt.entry), yt.entry
  url = yt.entry[0].link[0].href
  print url
